@@ -8,6 +8,11 @@ class Image(ExternalElement):
     url: Optional[str] = None
     path: Optional[str] = None
 
+    def asSerializationString(self) -> str:
+        return f"[mirai:image:{self.imageId}]"
+
 class FlashImage(Image):
     type = "FlashImage"
-    pass
\ No newline at end of file
+
+    def asSerializationString(self) -> str:
+        return f"[mirai:flash:{self.imageId}]"
\ No newline at end of file
